// Detects mouse driver, moves mouse pointer to the top left corner

procedure DetectMouse;
begin
  ButtonCount := SysTVDetectMouse;
end;

// Shows mouse pointer
procedure ShowMouse;
begin
  SysTVShowMouse;
end;

// Hides mouse pointer

procedure HideMouse;
begin
  SysTVHideMouse;
end;

// Initializes Turbo Vision's event manager by setting event mask and
// showing the mouse. Called automatically by TApplication.Init.
procedure InitEvents;
begin
  if ButtonCount <> 0 then
  begin
    DownButtons := 0;
    LastDouble := False;
    LastButtons := 0;       // Assume that no button is pressed
    SysTVInitMouse(MouseWhere.X, MouseWhere.Y);
    MouseEvents := True;
  end;
end;

// Terminates Turbo Vision's event manager and hides the mouse. Called
// automatically by TApplication.Done.
procedure DoneEvents;
begin
  if ButtonCount <> 0 then
  begin
    SysTVDoneMouse(False);
    MouseEvents := False;
  end;
end;

// Checks whether a mouse event is available by polling the mouse event
// queue maintained by OS/2. If a mouse event has occurred, Event.What
// is set to evMouseDown, evMouseUp,evMouseMove, or evMouseAuto;
// Event.Buttons is set to mbLeftButton or mbRightButton;
// Event.Double is set to True or False;
// Event.Where is set to the mouse position in global coordinates.
// If no mouse events are available, Event.What is set to evNothing.
// GetMouseEvent is called by TProgram.GetEvent.
procedure GetMouseEvent(var Event: TEvent);
var
  SysMouseEvent: TSysMouseEvent;
  CurTicks: Word;
  B: Byte;

procedure StoreEvent(MouWhat: Word);
begin
  LastButtons := MouseButtons;
  MouseWhere.X := SysMouseEvent.smePos.X;
  MouseWhere.Y := SysMouseEvent.smePos.Y;
  with Event do
  begin
    What := MouWhat;
    Buttons := MouseButtons;
    Double  := LastDouble;
    Where.X := SysMouseEvent.smePos.X;
    Where.Y := SysMouseEvent.smePos.Y;
  end;
end;

// GetMouseEvent body

begin
  if not MouseEvents then
    Event.What := evNothing
  else
    begin
      if not SysTVGetMouseEvent(SysMouseEvent) then
        begin
          MouseButtons := LastButtons;
          SysMouseEvent.smeTime := SysSysMsCount;
          SysMouseEvent.smePos.X := MouseWhere.X;
          SysMouseEvent.smePos.Y := MouseWhere.Y;
        end
      else
        begin
          if MouseReverse then
          begin
            B := 0;
            if (SysMouseEvent.smeButtons and $0001) <> 0 then
              Inc(B, $0002);
            if (SysMouseEvent.smeButtons and $0002) <> 0 then
              Inc(B, $0001);
            SysMouseEvent.smeButtons := B;
          end;
          MouseButtons := SysMouseEvent.smeButtons;
        end;
      // ms -> ticks: 1 DOS timer tick = 55ms
      CurTicks  := SysMouseEvent.smeTime div 55;
      // Process mouse event
      if (LastButtons <> 0) and (MouseButtons = 0) then
        StoreEvent(evMouseUp) // button is released
      else
        if LastButtons = MouseButtons then
          begin
            if (SysMouseEvent.smePos.Y <> MouseWhere.Y) or (SysMouseEvent.smePos.X <> MouseWhere.X) then
              StoreEvent(evMouseMove)
            else
              if (MouseButtons <> 0) and ((CurTicks - AutoTicks) >= AutoDelay) then
                begin
                  AutoTicks := CurTicks;
                  AutoDelay := 1;
                  StoreEvent(evMouseAuto);
                end
              else
                StoreEvent(evNothing);
          end
        else // CurButton <> 0, LastButton = 0
          begin
            LastDouble := False;
            if (MouseButtons = DownButtons) and (SysMouseEvent.smePos.Y = DownWhere.Y) and (SysMouseEvent.smePos.X = DownWhere.X)
              and ((CurTicks - DownTicks) < DoubleDelay) then
                LastDouble := True;
            DownButtons := MouseButtons;
            DownWhere.Y := SysMouseEvent.smePos.Y;
            DownWhere.X := SysMouseEvent.smePos.X;
            DownTicks   := CurTicks;
            AutoTicks   := CurTicks;
            AutoDelay   := RepeatDelay;
            StoreEvent(evMouseDown);
          end;
    end;
end;

procedure UpdateMouseWhere;
begin
  SysTVUpdateMouseWhere(MouseWhere.X, MouseWhere.Y);
end;

////Keyboard
// Checks whether a keyboard event is available. If a key has been
// pressed, Event.What is set to evKeyDown and Event.KeyCode is set to
// the scan code of the key. Otherwise, Event.What is set to evNothing.
// GetKeyEvent is called by TProgram.GetEvent.

procedure InitKeyboard;
begin
  SysTVKbdInit;
end;

procedure GetKeyEvent(var Event: TEvent);
var
  I: Integer;
  SysKeyEvent: TSysKeyEvent;
// Keyboard scan codes
const
  scSpace    = $39;  scIns      = $52;  scDel      = $53;
  scBack     = $0E;  scUp       = $48;  scDown     = $50;
  scLeft     = $4B;  scRight    = $4D;  scHome     = $47;
  scEnd      = $4F;  scPgUp     = $49;  scPgDn     = $51;
  scCtrlIns  = $92;  scCtrlDel  = $93;  scCtrlUp   = $8D;
  scCtrlDown = $91;  kbShift    = kbLeftShift + kbRightShift;

type
  KeyTransEntry = record
    Scan:  Byte;
    Shift: Byte;
    Code:  SmallWord;
  end;

const
  KeyTranslateTable : array [1..15] of KeyTransEntry =
    (( Scan: scSpace   ; Shift: $08 ; Code: kbAltSpace    ), // Alt-Space
     ( Scan: scIns     ; Shift: $04 ; Code: kbCtrlIns     ), // Ctrl-Ins
     ( Scan: scCtrlIns ; Shift: $04 ; Code: kbCtrlIns     ), // Ctrl-Ins
     ( Scan: scIns     ; Shift: $01 ; Code: kbShiftIns    ), // Shift-Ins
     ( Scan: scIns     ; Shift: $02 ; Code: kbShiftIns    ), // Shift-Ins
     ( Scan: scIns     ; Shift: $03 ; Code: kbShiftIns    ), // Shift-Ins
     ( Scan: scDel     ; Shift: $04 ; Code: kbCtrlDel     ), // Ctrl-Del
     ( Scan: scCtrlDel ; Shift: $04 ; Code: kbCtrlDel     ), // Ctrl-Del
     ( Scan: scDel     ; Shift: $01 ; Code: kbShiftDel    ), // Shift-Del
     ( Scan: scDel     ; Shift: $02 ; Code: kbShiftDel    ), // Shift-Del
     ( Scan: scDel     ; Shift: $03 ; Code: kbShiftDel    ), // Shift-Del
     ( Scan: scBack    ; Shift: $09 ; Code: kbAltShiftBack), // Alt-Shift-Backspace
     ( Scan: scBack    ; Shift: $0A ; Code: kbAltShiftBack), // Alt-Shift-Backspace
     ( Scan: scBack    ; Shift: $0B ; Code: kbAltShiftBack), // Alt-Shift-Backspace
     ( Scan: scBack    ; Shift: $08 ; Code: kbAltBack    )); // Alt-Backspace
begin
  with Event do
    if not SysTVGetKeyEvent(SysKeyEvent) then
      What := evNothing
    else
      begin
        What       := KeyDownMask;
        KeyCode    := SysKeyEvent.skeKeyCode;
        ShiftState := SysKeyEvent.skeShiftState;
        for I := Low(KeyTranslateTable) to High(KeyTranslateTable) do
          with KeyTranslateTable[I] do
          begin
            if (Scan = ScanCode) and ((Shift and ShiftState) = Shift) then
            begin
              KeyCode := Code;
              Break;
            end;
          end;
        if (CharCode = #$E0) and (ScanCode in
          [scUp,scDown,scLeft,scRight,scIns,scDel,scHome,scEnd,scPgUp,scPgDn,
          Hi(kbCtrlHome), Hi(kbCtrlEnd)  , Hi(kbCtrlPgUp), Hi(kbCtrlPgDn),
          Hi(kbCtrlLeft), Hi(kbCtrlRight),    scCtrlUp,       scCtrlDown]) then
            CharCode := #0;      // Grey Keys
        if KeyCode = $E00D then  // Grey Enter
          KeyCode := kbEnter;
        InfoPtr := nil;
      end;
end;

// Returns a byte containing the current Shift key state, as reported by
// the system. The return value contains a combination of the kbXXXX constants
// for shift states.

function GetShiftState: Byte;
var
  Handled: Boolean;
begin
  Handled := False;
  if @GetShiftStateHandler <> nil then
    Handled := GetShiftStateHandler(Result);
  if not Handled then
    Result := SysTVGetShiftState;
end;




